<?php
/*
Plugin Name: UPS Order Pickup Date based on Delivery Date
Description: Add-on plugin to calculate UPS pickup date based on the customer desired delivery date.
Version: 1.0
Author: PluginHive
Author URI: https://www.pluginhive.com/about/
Text Domain: ph-order-delivery-based-pickup
*/

if ( ! defined( 'ABSPATH' ) ) {
		exit; // Exit if accessed directly.
}	

//check if UPS_WooCommerce_Shipping exists
if ( !class_exists( 'UPS_WooCommerce_Shipping' ) ) {   
	add_action( 'admin_init', 'ph_ups_estimate_order_pickup_plugin_deactivate' );
	if ( ! function_exists( 'ph_ups_estimate_order_pickup_plugin_deactivate' ) ) {
		function ph_ups_estimate_order_pickup_plugin_deactivate() {
			  	if ( !class_exists( 'UPS_WooCommerce_Shipping' ) ){
		           deactivate_plugins( plugin_basename( __FILE__ ) );
		           wp_safe_redirect( admin_url('plugins.php') );    
			  	}
		}
	}
}

if( ! function_exists('ph_ups_order_delivery_based_estimate_activate') ) {
	function ph_ups_order_delivery_based_estimate_activate(){

		if ( !is_plugin_active('ups-woocommerce-shipping/ups-woocommerce-shipping.php') ){
			deactivate_plugins( basename( __FILE__ ) );
			wp_die( __("Oops! You tried installing the UPS estimate Order Pickup based on Delivery Date without activating UPS WooCommerce Shipping Plugin. Kindly activate <b>UPS WooCommerce Shipping Plugin</b> and then try again", "ph_residential_order_signature", 'ph-ups-order-delivery-based-pickup' ), "", array('back_link' => 1 ));
		}	
	}
}
register_activation_hook( __FILE__, 'ph_ups_order_delivery_based_estimate_activate' );

//add settings page for holidays
function ph_ups_order_delivery_setup_menu(){
	add_menu_page( 'Order Delivery Settings', 'Holiday Settings', 'manage_options', 'ph-delivery-settings', 'ph_delivery_settings' );
}
add_action('admin_menu', 'ph_ups_order_delivery_setup_menu'); 
 

function ph_delivery_settings(){
	if( ! class_exists('Ph_Estimate_holidays_Settings') )
		include_once( 'includes/class-ph-estimate-holidays-settings.php' );
	new Ph_Estimate_holidays_Settings; 
}

//to add settings url near plugin under installed plugin
function ph_estimate_delivery_holidays_actions_links( $links ) {

	$plugin_links = array(
		'<a href="' . admin_url( 'admin.php?page=' . 'ph-delivery-settings' ) . '">' . __('Holiday Settings', 'ph-ups-order-delivery-based-pickup') .'</a>',
	);
	return array_merge( $plugin_links, $links );
}
add_filter( 'plugin_action_links_' . plugin_basename( __FILE__ ), 'ph_estimate_delivery_holidays_actions_links' );


const PH_ORDER_DELIVERY_DATE_META_KEY  = '_orddd_timestamp';
const PH_ORDER_STORED_PACKAGE_META_KEY = '_wf_ups_stored_packages';
const PH_META_BOX_TITLE		 		= "UPS Pickup Date";
const PH_TEXT_DOMAIN				= "ph-ups-order-delivery-based-pickup";
const PH_BUFFER_DAYS_KEY			= "ph_ups_buffer_days_pickup";

if ( is_admin() ) { 
	add_action( 'add_meta_boxes', 'ph_add_admin_metabox', 15 );
	if ( isset( $_GET['ph_ups_order_shipment'] ) ) {
		add_action( 'init', 'ph_get_buffer_days_for_order', 15 );
	}

	if ( isset( $_POST['get_PDF'])){

		$order_id = $_POST['order_id'];
		$shipment_details = get_post_meta( $order_id, 'ph_ups_order_delivery_date', true );
		if( isset( $shipment_details['error'] )){
			exit;
		}
		require_once('includes/TCPDF-master/tcpdf.php');
		ob_start();
		$obj_pdf = new TCPDF('P', PDF_UNIT, PDF_PAGE_FORMAT, true, 'UTF-8', false);  
		$obj_pdf->SetCreator(PDF_CREATOR);  
		$obj_pdf->SetTitle("Estimated Shipment Dates");  
		$obj_pdf->SetHeaderData('', '', PDF_HEADER_TITLE, PDF_HEADER_STRING);  
		$obj_pdf->setHeaderFont(Array(PDF_FONT_NAME_MAIN, '', PDF_FONT_SIZE_MAIN));  
		$obj_pdf->setFooterFont(Array(PDF_FONT_NAME_DATA, '', PDF_FONT_SIZE_DATA));  
		$obj_pdf->SetDefaultMonospacedFont('helvetica');  
		$obj_pdf->SetFooterMargin(PDF_MARGIN_FOOTER);  
		$obj_pdf->SetMargins(PDF_MARGIN_LEFT, '5', PDF_MARGIN_RIGHT);  
		$obj_pdf->setPrintHeader(false);  
		$obj_pdf->setPrintFooter(false);  
		$obj_pdf->SetAutoPageBreak(TRUE, 10);  
		$obj_pdf->SetFont('helvetica', '', 12);  
		$obj_pdf->AddPage();
		 

		$content = '';
		$content .= '  
		<h1 align="center" style="font-size: 250%">Estimated Shipment Dates</h1><br /><br /><br />
		<h3 style="font-size: 200%">Order ID: ' . $shipment_details["order_id"] .'</h3><br /><br />
		<h3 style="font-size: 200%">Desired Delivery Date: ' . $shipment_details["delivery_date"] . ' </h1><br /><br /><br />
		<h3 style="font-size: 200%">Deliver to UPS: </h3><h1 style="color: red;font-size: 400%;">' . $shipment_details["pickup_date"] . '</h1><br /><br /><br />
		<h3 style="font-size: 200%">' . $shipment_details["days_in_transit"] . ' day(s) Delivery, ' . $shipment_details["buffer_days_value"] . ' buffer day(s).</h1><br /><br /><br />';
		
		ob_end_clean();
		$obj_pdf->writeHTML($content);  
		$obj_pdf->Output('estimated-shipping-dates.pdf', 'D');
		ob_end_flush();
		exit;
	}
}

function ph_get_buffer_days_for_order(){

	$post_id 		= isset( $_GET['post'] ) ? $_GET['post'] : '';
	$buffer_days	= isset( $_GET[ 'ph_ups_order_shipment' ] ) ? $_GET[ 'ph_ups_order_shipment' ] : '';
	
	update_post_meta( $post_id, PH_BUFFER_DAYS_KEY, $buffer_days );
	wp_redirect( admin_url( '/post.php?post='.$post_id.'&action=edit&'.PH_BUFFER_DAYS_KEY.'='.$buffer_days ) );
	exit;	
}


function ph_add_admin_metabox(){
	global $post;
	if ( !$post ) return false;
	if ( in_array( $post->post_type, array('shop_order') ) ) {
	
		// Shipment Tracking meta box for UPS.
		add_meta_box( 'PH_ups_order_delivery_metabox', __( PH_META_BOX_TITLE , PH_TEXT_DOMAIN ), 'ph_ups_order_delivery_metabox', 'shop_order', 'side', 'default' );
	}
}

function ph_ups_order_delivery_metabox(){
	global $post;

	$buffer_days_value 	= get_post_meta( $post->ID, PH_BUFFER_DAYS_KEY, true );
	?>
	
	<div class="add_label_id" >
		<strong>Enter buffer days</strong>
		<select id="<?php echo PH_BUFFER_DAYS_KEY; ?>" name="<?php echo PH_BUFFER_DAYS_KEY; ?>">
			<option <?php if( $buffer_days_value == "0"){echo "selected='selected'";} ?> value="0">0</option>
			<option <?php if( $buffer_days_value == "1"){echo "selected='selected'";} ?> value="1">1</option>
			<option <?php if( $buffer_days_value == "2"){echo "selected='selected'";} ?> value="2">2</option>
			<option <?php if( $buffer_days_value == "3"){echo "selected='selected'";} ?> value="3">3</option>
			<option <?php if( $buffer_days_value == "4"){echo "selected='selected'";} ?> value="4">4</option>
			<option <?php if( $buffer_days_value == "5"){echo "selected='selected'";} ?> value="5">5</option>
			<option <?php if( $buffer_days_value == "6"){echo "selected='selected'";} ?> value="6">6</option>
			<option <?php if( $buffer_days_value == "7"){echo "selected='selected'";} ?> value="7">7</option>
			<option <?php if( $buffer_days_value == "8"){echo "selected='selected'";} ?> value="8">8</option>
			<option <?php if( $buffer_days_value == "9"){echo "selected='selected'";} ?> value="9">9</option>
		</select>
		<span class="woocommerce-help-tip" data-tip="Enter the days you want to add to the final UPS pickup date."></span>
	</div>
	<?php
		$tracking_url = admin_url( '/?post='.( $post->ID ) );
	?>
		<a class="button button-primary ph_order_pickup tips" href="<?php echo $tracking_url; ?>" data-tip="<?php _e('Save the buffer days', PH_TEXT_DOMAIN); ?>"><?php _e('Save', PH_TEXT_DOMAIN); ?></a><hr style="border-color:#0074a2">
		
		<script type="text/javascript">
			jQuery("a.ph_order_pickup").on("click", function() {
			   location.href = this.href + '&ph_ups_order_shipment=' + jQuery('#ph_ups_buffer_days_pickup').val().replace(/ /g,'');
			   return false;
			});
		</script>
	<?php
		$shipment_date_details = ! empty( get_post_meta( $post->ID, 'ph_ups_order_delivery_date', true )) ? get_post_meta( $post->ID, 'ph_ups_order_delivery_date', true ) : null;
		if( (! isset( $shipment_date_details )) || isset ($shipment_date_details['error'])){
			return;
		}
	?>
		<form method='post'>
			<input type="hidden" id="order_id" name="order_id" value=<?php echo($post->ID);?>>
			<input type="submit" name="get_PDF" class="btn button-primary" value="Print UPS Pickup Date as PDF" />
		</form>
	<?php	
}

//Calculate pickup dates during rate request
if( ! function_exists('ph_ups_generate_estimated_pickup_date') ) {
	function ph_ups_generate_estimated_pickup_date( $request, $order, $shipment ) {
		// _wf_ups_stored_packages
		$package_details = get_post_meta($order->get_id(), PH_ORDER_STORED_PACKAGE_META_KEY, 1);

		$wf_shipping_obj = new WF_Shipping_UPS; 

		$shipping_address		= $order->get_address('shipping');
		$address_package    = array(
			'contents_cost'	=> 5,
			'destination'	=> array(
				'address'	=>	$shipping_address['address_1'].' '.$shipping_address['address_2'],
				'country'	=>	$shipping_address['country'],
				'state'		=>	$shipping_address['state'],
				'postcode'	=>	$shipping_address['postcode'],
				'city'		=>	$shipping_address['city'],

			),
		);

		$selected_service = '03';

		$rate_requests 		= $wf_shipping_obj->get_rate_requests( $package_details, $address_package );
		$days_in_transit 	= ph_response_get_transit_days_result( $wf_shipping_obj->get_result($rate_requests),'',$rate_requests, $selected_service );

		if( ! isset( $days_in_transit )) {
			$order_estimated_date = array(
				'error' => 'Unable to get Required Delivery days for shipment service( Ground ).',
			);
			$logger = wc_get_logger();
			$context = array( 'source' => 'ph-ups-delivery-pickup' );
			$logger->debug( json_encode($order_estimated_date), $context );
			update_post_meta( $order->get_id(), 'ph_ups_order_delivery_date', $order_estimated_date );
			return $request;
		}
		$buffer_days_value 	= !empty( get_post_meta( $order->get_id(), PH_BUFFER_DAYS_KEY, true )) ? get_post_meta( $order->get_id(), PH_BUFFER_DAYS_KEY, true ) : 0;
		;

		$order_timestamp = !empty( get_post_meta($order->get_id(), PH_ORDER_DELIVERY_DATE_META_KEY, 1) ) ? get_post_meta($order->get_id(), PH_ORDER_DELIVERY_DATE_META_KEY, 1) : null ;
		if( ! isset( $order_timestamp ) ){
			$order_estimated_date = array(
				'error' => 'Unable to get Desired Delivery date. ( Ground )',
			);
			$logger = wc_get_logger();
			$context = array( 'source' => 'ph-ups-delivery-pickup' );
			$logger->debug( json_encode($order_estimated_date), $context );
			update_post_meta( $order->get_id(), 'ph_ups_order_delivery_date', $order_estimated_date );
			return $request;
		}

		$total_days_in_transit = $days_in_transit + $buffer_days_value;
		$current_date = $order_timestamp;
		for( $i = 0; $i < $total_days_in_transit; $i++ ) {
			while( ph_is_weekend( $current_date ) || ph_is_holiday( $current_date ) ){
				$current_date = strtotime("-1 days", $current_date);
			}
			$current_date = strtotime("-1 days", $current_date);
		}
		while( ph_is_weekend( $current_date ) || ph_is_holiday( $current_date ) ){
			$current_date = strtotime("-1 days", $current_date);
		}

		$order_estimated_date = array(
			'order_id'		 => $order->get_id(),
			'delivery_date'	=> date('F j Y (l)', $order_timestamp),
			'pickup_date'	 => date('F j Y (l)', $current_date),
			'days_in_transit' => $days_in_transit,
			'buffer_days_value' => $buffer_days_value,
		);
		update_post_meta( $order->get_id(), 'ph_ups_order_delivery_date', $order_estimated_date );
		return $request;
	}
}
add_filter('wf_ups_shipment_confirm_request', 'ph_ups_generate_estimated_pickup_date', 10,3 );

function ph_is_weekend($date) {
	return (date('N', $date) >= 6 ? true : false);
}

function ph_is_holiday($date) {

	$holidays = get_option( 'ph_estimated_delivery_holiday');
	if( $holidays == false ) {
		return false;
	}

	foreach ($holidays as $key => $holiday) {
		// This date can be any English textual format  
		$date_from = strtotime(str_replace( '/','-', $holidays[$key]['from'] ) ); // Convert date to a UNIX timestamp  

		// This date can be any English textual format  
		$date_to = strtotime(str_replace( '/','-', $holiday['to'] )); // Convert date to a UNIX timestamp  
		
		// Loop from the start date to end date and output all dates inbetween  
		if( isset( $date_from ) && isset( $date_to )) {
			for ($i=$date_from; $i<=$date_to; $i+=86400) {  
				if( date('d-m-Y', $date) == date('d-m-Y', $i) ) {
					return true;
				}  
			}
		}
		
	}
	return false;
}

function ph_response_get_transit_days_result( $ups_response, $type='',$package=array(), $selected_service )
{
	//for freight response
	if( $type == 'json' ){
		$xml=json_decode($ups_response);
	}else{
		$xml = simplexml_load_string( preg_replace('/<\?xml.*\?>/','', $ups_response ) );
	}
	
	if ( ! $xml ) {
		return;
	}
	$rates = array();
	if ( ( property_exists($xml,'Response') && $xml->Response->ResponseStatusCode == 1)  || ( $type =='json' && !property_exists($xml,'Fault') ) ) {

		$xml = apply_filters('wf_ups_rate', $xml,$package);
		foreach ( $xml as $response ) {
			if($response->Service->Code == $selected_service ){
				if( isset($response->TimeInTransit->ServiceSummary->EstimatedArrival->BusinessDaysInTransit) ) {
						
					$days_in_transit 	= (string) $response->TimeInTransit->ServiceSummary->EstimatedArrival->BusinessDaysInTransit;
					// $current_time 		= clone $this->current_wp_time;
					return $days_in_transit;
				}
			}
		}
	}
	return;
}
